﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.XPath;
using System.Xml;
using System.Xml.Linq;

namespace Chapter_21
{
    class Program
    {
        string m_xmlFileName = "moneytalk.xml";

        static void Main()
        {
            var prog = new Program();

            prog.XmlReaderWriterDemo();

            prog.XPathDocumentDemo();

            prog.XmlDocumentDemo();

            prog.CreateLINQtoXMLDocument();

            prog.CreateLINQtoXMLDocumentWithAttributes();

            prog.CreateLINQtoXMLDocumentWithNamespace();

            prog.ReadXMLDocWithLINQtoXML();

            prog.QueryWithLINQtoXML();

            prog.ModifyDocWithLINQtoXML();

            Console.ReadKey();
        }

        private void ModifyDocWithLINQtoXML()
        {
            var doc = XDocument.Load("moneytalk.xml");

            doc.Element("moneyTalk").Add(
                new XElement("talk",
                    "Keep your wooden nickel.  It'll be worth something someday."));

            var talkElements =
                from talk in doc.Element("moneyTalk").Elements()
                where talk.Value.Contains("dime")
                select talk;

            talkElements.Single().Remove();

            talkElements =
                from talk in doc.Element("moneyTalk").Elements()
                where talk.Value.Contains("dollar")
                select talk;

            talkElements.Single().ReplaceWith(
                new XElement("talk", 
                    "A penny saved is too small, make it a buck."));

            doc.Save("moneytalk.xml");

            doc = XDocument.Load("moneytalk.xml");

            Console.WriteLine("LINQ to XML Doc: \n" + doc.ToString());
        }

        private void QueryWithLINQtoXML()
        {
            var doc = XDocument.Load("moneytalk.xml");

            var talkElements =
                from talk in doc.Element("moneyTalk").Elements()
                where talk.Value.Contains("dime")
                select talk;

            talkElements.ToList().ForEach(
                talkElement => Console.WriteLine(
                    "Talk Element: {0}", talkElement.Value));
        }

        private void ReadXMLDocWithLINQtoXML()
        {
            var doc = XDocument.Load("moneytalk.xml");

            Console.WriteLine("LINQ to XML Doc: \n" + doc.ToString());
        }

        private void CreateLINQtoXMLDocumentWithNamespace()
        {
            XNamespace moneyNames = "http://www.samspublishing.com/CSharp30Unleashed/XML";

            XDocument doc =
                new XDocument(
                    new XElement(moneyNames + "moneyTalk",
                        new XElement(moneyNames + "talk", "A penny saved is too small, make it a buck."),
                        new XElement(moneyNames + "talk", "Keep your wooden nickel.  It'll be worth something someday.")));

            Console.WriteLine("LINQ to XML Doc: \n" + doc.ToString());
        }

        private void CreateLINQtoXMLDocumentWithAttributes()
        {
            XDocument doc =
                new XDocument(
                    new XElement("moneyTalk",
                        new XElement("talk", 
                            new XAttribute("quote", "A penny saved is too small, make it a buck.")),
                        new XElement("talk", 
                            new XAttribute("quote", "Keep your wooden nickel.  It'll be worth something someday."))));

            Console.WriteLine("LINQ to XML Doc: \n" + doc.ToString());
        }

        private void CreateLINQtoXMLDocument()
        {
            XDocument doc =
                new XDocument(
                    new XElement("moneyTalk",
                        new XElement("talk", "A penny saved is too small, make it a buck."),
                        new XElement("talk", "Keep your wooden nickel.  It'll be worth something someday.")));

            Console.WriteLine("LINQ to XML Doc: \n" + doc.ToString());
        }
        
        /// <summary>
        /// shows how to edit with an XmlDocument object
        /// </summary>
        private void XmlDocumentDemo()
        {
            var xDoc = new XmlDocument();
            xDoc.Load(m_xmlFileName);

            var xPathNav = xDoc.CreateNavigator();

            xPathNav.MoveToFirstChild();
            xPathNav.MoveToNext();
            xPathNav.AppendChild(
                "<talk>Money isn't e...? Wait a minute - what planet are you from?</talk>");

            xPathNav.MoveToFirstChild();
            xPathNav.SetValue(xPathNav.Value.Replace("buck", "dollar"));

            xPathNav.MoveToNext();
            xPathNav.DeleteSelf();

            Console.WriteLine(xDoc.OuterXml);

            xDoc.Save(m_xmlFileName);
        }

        /// <summary>
        /// shows how to read from an XPathDocument
        /// </summary>
        private void XPathDocumentDemo()
        {
            var xPathDoc = new XPathDocument(m_xmlFileName);

            XPathNavigator xPathNav = xPathDoc.CreateNavigator();
            XPathNodeIterator xPathIter = xPathNav.Select("//talk");

            foreach (var node in xPathIter)
            {
                Console.WriteLine("Money Talk: {0}", node.ToString());
            }
        }

        /// <summary>
        /// shows how to use XmlReader and XmlWriter
        /// </summary>
        private void XmlReaderWriterDemo()
        {
            var money = new MoneyTalk
            {
                FileName = m_xmlFileName
            };

            money.WriteXML();
            money.ReadXML();
        }
    }
}
