﻿/// <summary>
/// Remoting Object Lifetime Demo.
/// </summary>
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Lifetime;
using System;
using BasicServer;
using System.Runtime.Remoting.Channels.Http;
class LeasingDemo : ISponsor
{
    ILease lease;

    // implement lease logic
    public void ImplementLease()
    {
        BasicRemotingServer brs = new BasicRemotingServer();

        lease = (ILease)brs.InitializeLifetimeService();
        lease.Register(this, new TimeSpan(0, 0, 3));

        PrintLeaseInfo();

        Console.WriteLine("\nServer Response: {0}",
            brs.GetServerResponse());
    }

    // ISponsor.Renewal - called to renew lease
    public TimeSpan Renewal(ILease myLease)
    {
        TimeSpan timeSpan = new TimeSpan(0, 0, 3);

        Console.WriteLine("\nLease Renewed.\n");

        PrintLeaseInfo();

        return timeSpan;
    }

    // print lease info
    void PrintLeaseInfo()
    {
        if (lease != null)
        {
            Console.WriteLine("Lease Info\n");
            Console.WriteLine("  CurrentLeaseTime: {0}", lease.CurrentLeaseTime);
            Console.WriteLine("  InitialLeaseTime: {0}", lease.InitialLeaseTime);
            Console.WriteLine("   RenewOnCallTime: {0}", lease.RenewOnCallTime);
            Console.WriteLine("SponsorshipTimeout: {0}", lease.SponsorshipTimeout);
        }
    }

    // entry point
    static void Main(string[] args)
    {
        ChannelServices.RegisterChannel(new HttpChannel(), false);

        LeasingDemo leaseDemo = new LeasingDemo();
        leaseDemo.ImplementLease();
    }
}
