using System;
using System.Threading;
using System.Diagnostics;

namespace OrderingClient
{
	/// <summary>
	/// Summary description for CustomOrderProcessor.
	/// </summary>
	public class CustomOrderProcessor : IDisposable
	{
		private PerformanceCounter orderCounter;

		private Random rand;

		public CustomOrderProcessor()
		{
			rand = new Random();

			CounterCreationDataCollection myCounters = new CounterCreationDataCollection();
			
			CounterCreationData myCounterCreationData = new CounterCreationData();
			myCounterCreationData.CounterName = "Order Count";
			myCounterCreationData.CounterHelp = "Displays the number of orders being processed.";
			myCounterCreationData.CounterType = PerformanceCounterType.NumberOfItems32;
			myCounters.Add(myCounterCreationData);
				
			if (PerformanceCounterCategory.Exists("Order Processor"))
			{
				PerformanceCounterCategory.Delete("Order Processor");
			}

			PerformanceCounterCategory.Create(
					"Order Processor",
					"OrderProcessor class counters",
					myCounters);

			orderCounter = new PerformanceCounter(
				"Order Processor", 
				"Order Count",
				false);

			orderCounter.RawValue = 0;
		}

		public int ProcessOrder()
		{
			Thread th = new Thread(new ThreadStart(doOrder));
			th.Start();

			CurNoOrders++;
			return 0;
		}

		public int CurNoOrders
		{
			get 
			{
				return (int)orderCounter.NextValue();
			}
			set 
			{
				orderCounter.RawValue = value;
			}
		}

		private void doOrder()
		{
			for (int delay = rand.Next(1000000);
				delay >= 0;
				delay--)
				;

			CurNoOrders--;
		}

		public void Dispose() 
		{
			PerformanceCounterCategory.Delete("Order Processor");
		}
	}
}
